import type { UserInDatabase } from "@interfaces/db/mariadb.interface";
import JwtService from "@services/authentication/jwt.service";
import { DatabasesService } from "@services/databases/databases.service";
import { HeaderUtil } from "@utils/headers.util";
import { HttpStatusCode } from "axios";
import type { NextFunction, Request, Response } from "express";

const db = new DatabasesService("OnlyDevs");

async function UserMiddleware(req: Request, res: Response, next: NextFunction) {
	const originToken = await HeaderUtil.getToken(req);
	if (!originToken) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "Unauthorized" });
	}
	const tokenPayload = await JwtService.verify(`${originToken}`);
	if (!tokenPayload || !tokenPayload.sub) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "Unauthorized" });
	}
	const UserFound = await db.getUserById(tokenPayload.sub);
	const User: UserInDatabase | undefined = UserFound[0] as UserInDatabase;
	if (!User) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "Unauthorized, you dont exist." });
	}
	if (User.email_activation) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "You should verify your email first." });
	}
	return next();
}

async function AdminMiddleware(
	req: Request,
	res: Response,
	next: NextFunction,
) {
	const originToken = await HeaderUtil.getToken(req);
	if (!originToken) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "Unauthorized" });
	}
	const tokenPayload = await JwtService.verify(`${originToken}`);
	if (!tokenPayload || !tokenPayload.sub) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "Unauthorized" });
	}
	const UserFound = await db.getUserById(tokenPayload.sub);
	const User: UserInDatabase | undefined = UserFound[0] as UserInDatabase;
	if (!User) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "Unauthorized, you dont exist." });
	}
	if (User.email_activation) {
		return res
			.status(HttpStatusCode.Forbidden)
			.json({ message: "You should verify your email first." });
	}

	const adminState = User.admin;
	console.log(adminState);

	if (!adminState) {
		return res
			.status(HttpStatusCode.PreconditionRequired)
			.json({ message: "Unauthorized, you are not an admin." });
	}
	return next();
}

export const AuthMiddleware = {
	user: UserMiddleware,
	admin: AdminMiddleware,
};
